package util;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.json.JSONObject;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class Worker {

	public JSONObject getShows(String band)
			throws ParserConfigurationException, SAXException, IOException,
			ParseException {

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();

		Document doc = db.parse("http://api.jambase.com/search?band=" + band
				+ "&apikey=dgzrjx4bp2n5yn2n8twqdmuy");
		doc.getTextContent();

		NodeList nl = doc.getElementsByTagName("event");
		JSONObject shows = new JSONObject();

		shows.put("band", band);

		for (int i = 0; i < nl.getLength(); i++) {
			Node node = nl.item(i);
			Hashtable<String, String> show = getTable(node);
			JSONObject jsonShow = toJson(show);
			if (jsonShow != null)
				shows.accumulate("show", jsonShow);

		}

		System.out.println(shows.toString());

		return shows;
	}

	private JSONObject toJson(Hashtable<String, String> show)
			throws IOException, ParseException {
		JSONObject object = new JSONObject();
		String city = show.get("venue_city");
		String state = show.get("venue_state");
		String zip = show.get("venue_zip");

		object.put("city", city);
		object.put("state", state);

		city = city.replaceAll(" ", "");
		zip = zip.replaceAll(" ", "_");

		String cordinatesLine = getCordinatesLine(city, state, zip);

		if (cordinatesLine == null)
			return null;
		String[] cordinates = cordinatesLine.split(",");
		String lat = cordinates[0];
		String lng = cordinates[1];

		object.put("lat", lat);
		object.put("lng", lng);

		// DateFormat df = new SimpleDateFormat("MM/dd/yy");
		// Date date = df.parse(show.get("event_date"));
		//
		// object.put("date", date);
		object.put("date", show.get("event_date"));
		object.put("place", show.get("venue_name"));

		// System.out.println(object.toString());

		return object;
	}

	private String getCordinatesLine(String city, String state, String zip)
			throws IOException {
		String url = "http://tinygeocoder.com/create-api.php?q=" + city + ","
				+ state;

		if (zip != null && !zip.isEmpty() && !zip.equalsIgnoreCase("na"))
			url = url + "," + zip;

		InputStream is = getInputStreamFromUrl(url);

		BufferedReader reader = new BufferedReader(new InputStreamReader(is));

		return reader.readLine();
	}

	private Hashtable<String, String> getTable(Node node) {
		Hashtable<String, String> itens = new Hashtable<String, String>();

		NodeList nodes = node.getChildNodes();

		// obtendo a date
		for (int i = 0; i < nodes.getLength(); i++) {
			Node subNode = nodes.item(i);
			if (subNode.getNodeName().equals("event_date")) {
				itens.put("event_date", subNode.getTextContent());
				break;
			}
		}

		// obtendo a tag venue
		NodeList venue = null;
		for (int i = 0; i < nodes.getLength(); i++) {
			Node subNode = nodes.item(i);
			if (subNode.getNodeName().equals("venue")) {
				venue = subNode.getChildNodes();
			}
		}

		// obtendo todas as infos da tag venue
		for (int i = 0; i < venue.getLength(); i++) {
			Node subNode = venue.item(i);
			itens.put(subNode.getNodeName(), subNode.getTextContent());
		}

		return itens;
	}

	// Apenas para testes
	public static void main(String[] args) {
		try {
			try {
				new Worker().getShows("Metallica");
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private InputStream getInputStreamFromUrl(String urlAction) {
		HttpURLConnection conn = null;
		BufferedInputStream bis = null;

		try {
			URL url = getUrl(urlAction);
			if (url == null)
				return null;

			conn = (HttpURLConnection) url.openConnection();
			conn.setRequestProperty("Request-Method", "GET");
			conn.connect();

			bis = new BufferedInputStream(conn.getInputStream());

			// copiando para poder fechar conexão
			int c;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			while ((c = bis.read()) != -1) {
				baos.write(c);
			}
			ByteArrayInputStream bais = new ByteArrayInputStream(baos
					.toByteArray());
			return bais;

		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (bis != null)
					bis.close();
			} catch (IOException e) {

			}
			if (conn != null) {
				conn.disconnect();
			}
		}
		return null;
	}

	private URL getUrl(String urlString) {
		URL url = null;
		try {
			url = new URL(urlString);
			return url;
		} catch (MalformedURLException e) {
			return null;

		}
	}

}
